from tqdm import tqdm
import requests
import os
import click
from urllib.parse import unquote

def download_file(link, path):
    maximum_retries = 3
    
    filename = os.path.basename(link)
    filename = unquote(filename)

    click.echo(click.style(f"{filename}", fg="green"))
    with open(f"{path}/{filename}", 'wb') as f:
        download_size = 0
        while maximum_retries > 0:
            requests.adapters.HTTPAdapter(max_retries=maximum_retries)
            response = requests.get(link, stream=True, headers={'Accept-Encoding': None, 'Content-Encoding': 'gzip'})
            download_size = response.headers.get('content-length')
            if download_size is None and maximum_retries > 0:
                maximum_retries -= 1
            else:
                break
        pbar = tqdm(
            total=int(download_size),
            initial=0,
            unit='B',
            unit_scale=True,
            position=0,
            leave=True)
        for chunk in response.iter_content(chunk_size=1024):
            if chunk:
                f.write(chunk)
                pbar.set_description("Downloading... ")
                pbar.update(len(chunk))
        pbar.close()
        print("\n")



import os
import sys
import urllib.parse
import click

import requests
from bs4 import BeautifulSoup as bs
import re

from lged import config
from lged.settings import param_values
from lged.downloader import download_file

sys.path.append(os.getcwd() + '/..')

class App:
    def __init__(self, **kwargs):
        self.search_config_url = None
        self.search_url = None
        self.keyword = kwargs['keyword']
        self.view = kwargs['view']
        self.page_num = kwargs['page_num']
        self.search_field = kwargs['search_field']
        self.sort = kwargs['sort']
        self.sortmode = kwargs['sortmode']
        self.download_path = kwargs['download'] if kwargs['download'] != "" else config.download_path
            
    def run(self):
        click.echo(click.style(f"LGED is running...", fg="green"))

        self.configSearchURL()

        self.loadEBooks()
    
    def configSearchURL(self):
        params = {}
        params['req'] = self.keyword
        params['phrase'] = 1
        params['res'] = 25

        if self.view in param_values['view']:
            params['view'] = self.view
        else:
            click.echo(click.style(f"The view param is wrong value.", fg="red"))

        if self.search_field in param_values['search_field']:
            params['column'] = self.search_field
        else:
            click.echo(click.style(f"The search_field param is wrong value.", fg="red"))

        if self.sort in param_values['sort']:
            params['sort'] = self.sort
        else:
            click.echo(click.style(f"The sort param is wrong value.", fg="red"))

        if self.sortmode in param_values['sortmode']:
            params['sortmode'] = self.sortmode
        else:
            click.echo(click.style(f"The sortmode param is wrong value.", fg="red"))        
        
        self.search_config_url = f"{config.site_url}?{urllib.parse.urlencode(params)}"
        
    def loadEBooks(self):
        book_count = 1
        download_book_number = 25 * self.page_num
        for page in range(1, self.page_num + 1):
            params = {}
            params['page'] = page
            self.search_url = f"{self.search_config_url}&{urllib.parse.urlencode(params)}"
            print(f"{self.search_url}\n")

            try:
                page_data = requests.get( self.search_url )
                soup = bs(page_data.content, "html.parser")

                ebook_num_element = soup.find("font", string=re.compile("files found"))
                total_number = 0
                if ebook_num_element != None:
                    temp = ebook_num_element.text.split("|")[0]
                    temp = re.findall(r'\d+', temp)
                    numbers = list(map(int, temp))
                    total_number = numbers[0] if len(numbers) > 0 else 0

                total_number = total_number if total_number < download_book_number else download_book_number
                
                ebook_list = soup.find_all("tr", valign="top")
                for index, item in enumerate(ebook_list):
                    if index == 0:
                        continue

                    click.echo(click.style(f"Book: {book_count}/{total_number} - {page} page", fg="green"))
                    temp = item.find(href=re.compile("book"))
                    if temp is not None:
                        ebook_link = temp.get('href')
                        if ebook_link != "":
                            self.loadEbook(ebook_link)
                    
                    book_count = book_count + 1

            except Exception as e:
                click.echo(click.style(f"Error fetching the page!\n{e}", fg="red"))
    
    def loadEbook(self, ebook):
        ebook_link = f"{config.site_url}/{ebook}"
        click.echo(click.style(f"Loading ebook page - {ebook_link}", fg="green"))

        try:
            ebook_page = requests.get(ebook_link)
            soup = bs(ebook_page.content, "html.parser")
            tr_list = soup.find_all("tr", valign="top")
            if len(tr_list) > 0:
                item = tr_list[1]
                a_tag = item.find("a")
                ebook_detail_link = a_tag.get('href')
                if ebook_detail_link != "":
                    self.downloadEbook(ebook_detail_link)
        except Exception as e:
            click.echo(click.style(f"Error loading ebook page!\n{e}", fg="red"))

    def downloadEbook(self, ebook):
        click.echo(click.style(f"Loading ebook detail page - {ebook}", fg="green"))

        try:
            ebook_detail_page = requests.get(ebook)
            soup = bs(ebook_detail_page.content, "html.parser")
            a_tag = soup.find("a", string="GET")
            if a_tag is not None:
                file_link = a_tag.get('href')
                download_file(file_link, self.download_path)
        except Exception as e:
            click.echo(click.style(f"Error loading ebook detail page!\n{e}", fg="red"))


import argparse
from lged.app import App
from lged.settings import search_keys

def parse_command_line_args():
    parser = argparse.ArgumentParser(description="parse libgen.rs search parameters")
    parser.add_argument('--keyword', type=str, default=search_keys['keyword'], nargs='?', required=True,
                        help="""
                        enter search keys separated by a single space. If the keyword is more
                        than one word, wrap the keyword in double quotes.
                        """)
    parser.add_argument('--view', type=str, default=search_keys['view'], nargs='?',
                        help="""
                        enter view mode. You can check the view mode in the site.
                        The default value is "simple".
                        """)
    parser.add_argument('--page_num', type=int, default=search_keys['page_num'], nargs='?',
                        help="""
                        enter number of pages to download. The default value is 1.
                        """)
    parser.add_argument('--search_field', type=str, default=search_keys['search_field'], nargs='?',
                        help="""
                        enter search field. You can check it in the site.
                        The default value is "title".
                        """)    
    parser.add_argument('--sort', type=str, default=search_keys['sort'], nargs='?',
                        help="""
                        enter sort field. You can check the fields in the site.
                        The default value is "year".
                        """)
    parser.add_argument('--sortmode', type=str, default=search_keys['sortmode'], nargs='?',
                        help="""
                        enter sort mode: DESC or ASC. The default value is DESC.
                        """)
    parser.add_argument('--download', type=str, default=search_keys['download'], nargs='?',
                        help="""
                        enter download path. The default value is Downloads directory.
                        """)
    
    return vars(parser.parse_args())


def main():
    search_keys = parse_command_line_args()

    app = App(**search_keys)
    app.run()


search_keys = { 
    "keyword"           : "",

    # View
    # URL param: view
    # values: simple, detailed
    # Titles: Simple, Detailed Search with mask
    "view"              : "simple",

    # The number of pages
    # URL param: res
    "page_num"          : 1,

    # Search in fields
    # URL param: column
    # values: def, title, author, series, publisher, year, identifier, language, md5, tags, extension
    # Titles: The column set default, Title, Author(s), Series, Publisher, Year, ISBN, Language, MD5, Tags, Extension
    "search_field"      : "title",

    # Sort by
    # URL param: sort
    # values: id, author, title, publisher, year, pages, language, filesize, extension
    # Titles: ID, Author(s), Title, Publisher, Year, Pages, Language, Size, Extension
    "sort"              : "year",

    # Sort mode
    # URL param: sortmode
    # values: ASC, DESC
    "sortmode"          : "DESC",

    # Download path
    "download"          : ""
}

param_values = {
    "view": ["simple", "detailed"],
    "page_num": [25, 50, 100],
    "search_field": ["def", "title", "author", "series", "publisher", "year", "identifier", "language", "md5", "tags", "extension"],
    "sort": ["id", "author", "title", "publisher", "year", "pages", "language", "filesize", "extension"],
    "sortmode": ["ASC", "DESC"]
}